/*
  EleKin_v
    Electrokinetics - Electric scalar potential v formulation
  Thermal_T
    Thermal model - Temperature formulation
*/
// Patrick Dular

/* I N P U T
   ---------

  GlobalGroup :
  -----------
  (Extension '_Ele' is for Electric problem)
  (Extension '_The' is for Thermal problem)

    Domain_Ele               Whole electric domain
    DomainC_Ele              Conducting regions

    Domain_The               Whole thermal domain
    DomainJoule_The          Thermal regions with Joule losses
    DomainQ_The              Thermal regions with additional volume source
    SurfaceConv_The          All convection surfaces
    DomainTot_The            

  Function :
  --------
    sigma[]                  Electric conductivity
                               argument $1: Temperature

    lambda[]                 Thermal conductivity
    roc[]                    Calorific capacity
    h[]                      Convection coefficient
    TemperatureConv[]        Exterior temperature for convection
    Q_Vol[]                  Heat source volume density

  Constraint :
  ----------
    ElectricScalarPotential  Fixed electric scalar potential
                             (classical boundary condition)
    GlobalElectricPotential  Fixed global electric scalar potential
                             (for perfect electrodes)

    Temperature              Fixed temperature
                             (classical boundary condition)

  Physical constants :
  ------------------
*/

    eps0 = 8.854187818e-12 ;

/* O U T P U T
   -----------

  PostQuantities :
  --------------
   v : Electric scalar potential
   e : Electric field
   j : Electric current density

   T : Temperature

*/

/* --------------------------------------------------------------------------*/

Group {
  DefineGroup[ Domain_Ele, DomainC_Ele ] ;
}

Function {
  DefineFunction[ sigma ] ;
}


Group {
  DefineGroup[ Domain_The, DomainJoule_The, DomainQ_The, SurfaceConv_The ] ;
}

Function {
  DefineFunction[ lambda, rhoc, h, TemperatureConv, Q_Vol ] ;
  DefineFunction[ Time_DTime, Time_Theta ] ;
  DefineConstant[ Time_Time0=0., Time_TimeMax=1. ] ;
}


/* --------------------------------------------------------------------------*/

FunctionSpace {
  { Name Hgrad_v_EleKin ; Type Form0 ;
    BasisFunction {
      // v = v  s
      //      n  n
      { Name sn ; NameOfCoef vn ; Function BF_Node ;
        Support DomainC_Ele ; Entity NodesOf[ All ] ; }
    }
    Constraint {
      { NameOfCoef vn ;
        EntityType NodesOf ; NameOfConstraint ElectricScalarPotential ; }

      { NameOfCoef vn ;
        EntityType NodesOf ; NameOfConstraint GlobalElectricPotential ; }
    }
  }
}

Group { // Whole thermal domain with convection surfaces
  DomainTot_The = Region[ {Domain_The, SurfaceConv_The} ] ;
}
FunctionSpace {
  { Name Hgrad_T; Type Form0 ;
    BasisFunction {
      { Name sn ; NameOfCoef Tn ; Function BF_Node ;
        Support DomainTot_The ; Entity NodesOf[All] ; }
    }
    Constraint {
      { NameOfCoef Tn ; EntityType NodesOf ; NameOfConstraint Temperature ; }
    }
  }
}


Formulation {
  { Name Electrokinetics_v ; Type FemEquation ;
    Quantity {
      { Name v ; Type Local ; NameOfSpace Hgrad_v_EleKin ; }
      { Name T ; Type Local ; NameOfSpace Hgrad_T ; }
    }
    Equation {
      Galerkin { [ sigma[{T}] * Dof{d v} , {d v} ] ;
                 In DomainC_Ele ;
                 Jacobian Vol ; Integration GradGrad ; }
    }
  }
}

Formulation {
  { Name Thermal_T ; Type FemEquation ;
    Quantity {
      { Name T ; Type Local ; NameOfSpace Hgrad_T ; }
      { Name v ; Type Local ; NameOfSpace Hgrad_v_EleKin ; }
    }
    Equation {

      Galerkin { [ lambda[{T}]  * Dof{d T} , {d T} ] ;
                 In Domain_The; Integration GradGrad ; Jacobian Vol ; }

      Galerkin { DtDof[ rhoc[{T}]  * Dof{T} , {T} ] ;
                 In Domain_The; Integration GradGrad ; Jacobian Vol ; }

      Galerkin { [ - sigma[{T}] * SquNorm[{d v}] , {T} ] ; // Joule source
                 In DomainJoule_The ; Integration GradGrad ; Jacobian Vol ; }

/*    Galerkin { [ -Q_Sur[] , {T} ] ; 
                 In SurFlux_The ; Integration I1; Jacobian JSur;  } */

      Galerkin { [ -Q_Vol[{T}] , {T} ] ; // Additional volume source
                 In DomainQ_The ; Integration GradGrad ; Jacobian Vol ; }

      Galerkin { [ h[] * Dof{T} , {T} ] ; // Convection boundary condition
                 In SurfaceConv_The ; Integration Sur; Jacobian Sur ; }
      Galerkin { [ -h[] * TemperatureConv[] , {T} ] ;
                 In SurfaceConv_The ; Integration Sur; Jacobian Sur ; }
    }
  }

}


Resolution {
  { Name EleKin_v ; // not valid if T is in Formulation Electrokinetics_v
    System {
      { Name Sys_EleKin ; NameOfFormulation Electrokinetics_v ; }
    }
    Operation {
      Generate Sys_EleKin ; Solve Sys_EleKin ; SaveSolution Sys_EleKin ;
    }
  }

  { Name EleKinThe_v ; // statics for EleKin, then time domain for The
    System {
      { Name Sys_EleKin ; NameOfFormulation Electrokinetics_v ; }
      { Name Sys_The ; NameOfFormulation Thermal_T ; }
    }
    Operation {

      InitSolution Sys_The ; SaveSolution Sys_The ;
      
      Generate Sys_EleKin ; Solve Sys_EleKin ; SaveSolution Sys_EleKin ;
      
      SetTime Time_Time0 ;
      TimeLoopTheta{
        Time0 Time_Time0 ; TimeMax Time_TimeMax ;
        DTime Time_DTime[] ; Theta Time_Theta[] ;
        Operation {
	  Generate Sys_The ; Solve Sys_The ; SaveSolution Sys_The ;
        }
      }
    }
  }

  { Name Thermal_Time ; // not valid if v is in Formulation Thermal_T,
  // i.e., DomainJoule_The must be empty and only DomainQ_The is considered
    System {
      { Name Sys_The ; NameOfFormulation Thermal_T ; }
    }
    Operation {

      InitSolution Sys_The ; SaveSolution Sys_The ;
      
      SetTime Time_Time0 ;
      TimeLoopTheta{
        Time0 Time_Time0 ; TimeMax Time_TimeMax ;
        DTime Time_DTime[] ; Theta Time_Theta[] ;
        Operation {
	  Generate Sys_The ; Solve Sys_The ; SaveSolution Sys_The ;
        }
      }
    }
  }

  { Name Thermal_SteadyState ; // not valid if v is in Formulation Thermal_T,
  // i.e., DomainJoule_The must be empty and only DomainQ_The is considered
    System {
      { Name Sys_The ; NameOfFormulation Thermal_T ; }
    }
    Operation {

      InitSolution Sys_The ;
      
      IterativeLoop{
        NbrMaxIteration NL_NbrMax ; RelaxationFactor NL_Relax[] ; Criterion NL_Eps ;
        Operation {
	  GenerateJac Sys_The ; SolveJac Sys_The ;
        }
      }
      SaveSolution Sys_The ;
    }
  }


  { Name EleKinTheNL_v ; // with nonlinear iteration at each time step
    System {
      { Name Sys_EleKin ; NameOfFormulation Electrokinetics_v ; }
      { Name Sys_The ; NameOfFormulation Thermal_T ; }
    }
    Operation {
      SetTime Time_Time0 ;
      InitSolution Sys_The ; SaveSolution Sys_The ;
      Generate Sys_EleKin ; Solve Sys_EleKin ; SaveSolution Sys_EleKin ;
      
      TimeLoopTheta{
        Time0 Time_Time0 ; TimeMax Time_TimeMax ;
        DTime Time_DTime[] ; Theta Time_Theta[] ;
        Operation {
	  Generate Sys_The ; Solve Sys_The ; SaveSolution Sys_The ;
	  Generate Sys_EleKin ; Solve Sys_EleKin ; SaveSolution Sys_EleKin ;
        }
      } // An IterativeLoop could be added in the TimeLoopTheta...
    }
  }

}


PostProcessing {
  { Name EleKin_v ; NameOfFormulation Electrokinetics_v ;
    PostQuantity {
      { Name v ; Value { Term { [ {v} ]              ; In DomainC_Ele ; } } }
      { Name e ; Value { Term { [ -{d v} ]           ; In DomainC_Ele ; } } }
      { Name j ; Value { Term { [ -sigma[{T}] * {d v} ] ; In DomainC_Ele ; } } }
      { Name sigma ; Value { Term { [ sigma[{T}] ] ; In DomainC_Ele ; } } }
    }
  }
}

PostProcessing {
  { Name Thermal_T ; NameOfFormulation Thermal_T ;
    PostQuantity {
      { Name T ; Value { Term { [ {T} ] ; In Domain_The ; } } }
    }
  }
}


/* --------------------------------------------------------------------------*/
/* --------------------------------------------------------------------------*/
