/*
  EleSta_v
    Electrostatics - Electric scalar potential v formulation
*/
// Patrick Dular

/* I N P U T
   ---------

  GlobalGroups :  (Extension '_Ele' is for Electric problem)
  ------------
    Domain_Ele               Whole electric domain
    DomainCC_Ele             Nonconducting regions
    DomainC_Ele              Conducting regions (not used)

  Function :
  --------
    epsr[]                   Relative permittivity

  Constraint :
  ----------
    ElectricScalarPotential  Fixed electric scalar potential
                             (classical boundary condition)
    GlobalElectricPotential  Fixed global electric scalar potential
                             (for perfect conductors)

  Physical constants :
  ------------------
*/

    eps0 = 8.854187818e-12 ;

/* O U T P U T
   -----------

  PostQuantities :
  --------------
   v : Electric scalar potential
   e : Electric field
   d : Electric flux density

*/


/* --------------------------------------------------------------------------*/

Group {
  DefineGroup[ Domain_Ele, DomainCC_Ele, DomainC_Ele ] ;
}

Function {
  DefineFunction[ epsr ] ;
}


/* --------------------------------------------------------------------------*/

FunctionSpace {
  { Name Hgrad_v_Ele ; Type Form0 ;
    BasisFunction {
      // v = v  s   ,  for all nodes
      //      n  n
      { Name sn ; NameOfCoef vn ; Function BF_Node ;
        Support DomainCC_Ele ; Entity NodesOf[ All ] ; }
    }
    Constraint {
      { NameOfCoef vn ;
        EntityType NodesOf ; NameOfConstraint ElectricScalarPotential ; }
      { NameOfCoef vn ;
        EntityType NodesOf ; NameOfConstraint GlobalElectricPotential ; }
    }
  }
}


Formulation {
  { Name Electrostatics_v ; Type FemEquation ;
    Quantity {
      { Name v ; Type Local ; NameOfSpace Hgrad_v_Ele ; }
    }
    Equation {
      Galerkin { [ epsr[] * Dof{d v} , {d v} ] ;
                 In DomainCC_Ele ;
                 Jacobian Vol ; Integration GradGrad ; }
    }
  }
}


Resolution {
  { Name EleSta_v ;
    System {
      { Name Sys_Ele ; NameOfFormulation Electrostatics_v ; }
    }
    Operation { Generate Sys_Ele ; Solve Sys_Ele ; SaveSolution Sys_Ele ; }
  }
}


PostProcessing {
  { Name EleSta_v ; NameOfFormulation Electrostatics_v ;
    PostQuantity {
      { Name v ; Value { Term { [ {v} ]                 ; In DomainCC_Ele ; } } }
      { Name e ; Value { Term { [ -{d v} ]              ; In DomainCC_Ele ; } } }
      { Name d ; Value { Term { [ -eps0*epsr[] *{d v} ] ; In DomainCC_Ele ; } } }

      { Name em ; Value { Term { [ Norm[{d v}] ]        ; In DomainCC_Ele ; } } }
    }
  }
}


/* --------------------------------------------------------------------------*/
/* --------------------------------------------------------------------------*/
