/* Electrostatic problem: Microstrip line */

/* -------------------------------------------------------------------
   File "mStrip_GQ.pro"

   This file defines the problem dependent data structures for
   an electrostatic problem.

   Two resolutions:

   1. Only local quantities (1)
   To compute the solution: getdp mStrip_GQ -msh mStrip.msh -solve EleSta_v
   To compute post-results: getdp mStrip_GQ -msh mStrip.msh -pos Map_v Map_e Cut_e

   2. coupling with global quantities (voltages and charges) (2)
   To compute the solution: getdp mStrip_GQ -msh mStrip.msh -solve EleSta_vf
   To compute post-results: getdp mStrip_GQ -msh mStrip.msh -pos Map_vf Map_ef Cut_ef
                         or getdp mStrip_GQ -msh mStrip.msh -pos Val_Q
   ------------------------------------------------------------------- */

/* Geometry (2D) (1/2 structure by symmetry): in file "mStrip.geo"
                                        SurfInf
                                      /     /
   +------------------------------------+  /
   |                                    | /
   |                      Air           |/
   |                                    |   Mesh: in file "mStrip.msh"
   |          Line                      |
   |        /  /  /                     |     2D elements in:
   +-------/---+ /                      |       Air, Diel1
          /    |-                       |
   +-----------+------------------------+     1D elements in:
   |                      Diel1         |       Line, Ground, SurfInf
   |                                    |
   +------------------------------------+
                               \
                                 Ground
*/


Group {

  // Elementary groups
  Air = # 101 ; Diel1 = # 111 ;
  Ground = # 120 ; Line = # 121 ; SurfInf = # 130 ;

  // Global groups
  DomainCC_Ele    = Region[ {Air, Diel1} ] ;
  SkinDomainC_Ele = Region[ {Line} ] ; // (2), i.e. only used with global quantities

}


/* --------------------------------------------------------------------------*/

Function {

  // Relative permittivity piecewise defined in elementary groups
  epsr [ Air ] = 1. ;
  epsr [ Diel1 ] = 9.8 ;

  /* (2); the 'Length' (a constant) of the studied 2D structure
     enables a correct definition of the charge carried by parts of
     this structure (default value: 1 m) */

  Length = 1. ;  // Length of the line

}


/* --------------------------------------------------------------------------*/

Constraint {

  { Name ElectricScalarPotential ; Type Assign ;
    Case {
      { Region #{Ground, SurfInf} ; Value 0. ; }
    }
  }

  { Name GlobalElectricPotential ; Type Assign ;
    Case {
      //      { Region Line ; Value 1.e-3 ; }
    }
  }


  { Name GlobalElectricCharge ;
    Case {
      { Region Line ; Value 1.e-9 ; }
    }
  }

}


Include "../Jacobian_Lib.pro"
Include "../Integration_Lib.pro"

Include "../EleSta_v.pro"
Include "../EleSta_vf.pro"


PostOperation Map_v UsingPost EleSta_v {
  Print [ v, OnElementsOf DomainCC_Ele, File "mStrip_v.pos" ] ;
}

PostOperation Map_e UsingPost EleSta_v {
  Print [ e, OnElementsOf DomainCC_Ele, File "mStrip_e.pos" ] ;
}


PostOperation Map_vf UsingPost EleSta_vf {
  Print [ v, OnElementsOf DomainCC_Ele, File "mStrip_vf.pos" ] ;
}

PostOperation Map_ef UsingPost EleSta_vf {
  Print [ e, OnElementsOf DomainCC_Ele, File "mStrip_ef.pos" ] ;
}

e = 1.e-7 ;

PostOperation Cut_e UsingPost EleSta_v {
  Print [ e, OnLine { {e,e,0} {10.e-3,e,0} } {500}, File "Cut_e", Format Gnuplot ] ;
}

PostOperation Cut_ef UsingPost EleSta_vf {
  Print [ e, OnLine { {e,e,0} {10.e-3,e,0} } {500}, File "Cut_ef" ] ;
}



PostOperation Val_Q UsingPost EleSta_vf {
  Print [ Q, OnRegion SkinDomainC_Ele ] ;
  Print [ V, OnRegion SkinDomainC_Ele ] ;
  Print [ C, OnRegion SkinDomainC_Ele ] ;
}
