/* Electrokinetic problem: Plate with 3 holes */
// Patrick Dular

/* -------------------------------------------------------------------
   File "Plate.pro"

   This file defines the problem dependent data structures for
   an electrokinetic problem.

   Two resolutions:

   1. Only local quantities (1)
   To compute the solution: getdp Plate -solve EleKin_v
   To compute post-results: getdp Plate -pos Map_v Map_j Map_jg

   2. coupling with global quantities (voltages and currents) (2)
   To compute the solution: getdp Plate -solve EleKin_vf
   To compute post-results: getdp Plate -pos Map_vf_v Map_vf_j Map_vf_jg
                         or getdp Plate -pos Val_V
   ------------------------------------------------------------------- */

/* Geometry (2D): in file "Plate.geo"

        +------------------------------------------------------+
        |                              ___                     |
 Elec1  |       Plate                 /   \                    |
      \ |                            |     |                   |
        |               ___           \___/                    |
        |              /   \                                   |
        |             |     |                                  |
        |              \___/                ___                |
        |                                  /   \               |
        |                                 |     |              | \
        |                                  \___/               |  Elec0
        |                                                      |
        +------------------------------------------------------+

  Mesh: in file "Plate.msh"
    2D elements in: Plate
    1D elements in: Elec0, Elec1
*/

Group {
   
  /* Let's start by defining the interface (i.e. elementary groups)
     between the mesh file and GetDP (no mesh object is defined, so 
     the default mesh will be assumed to be in GMSH format and located 
     in "Plate.msh") */

  Plate = # 101 ;

  Elec0 = # 300 ;  Elec1 = # 301 ;

  /* We can then define global groups (used in "EleKin_v.pro" and "EleKin_vf.pro",
     the file containing the function spaces and formulations) */

  DomainC_Ele = Region[ {Plate} ] ;

  Electrode_Ele = Region[{Elec1}] ; // (2), i.e. only used with global quantities

}


/* --------------------------------------------------------------------------*/

Function {

  /* The electric conductivity (needed in the formulation) is piecewise
     defined in elementary groups */

  sigma [ Plate ] = 5.9e7 ;

  /* (2); the 'Length' (a constant) of the studied 2D structure
     enables a correct definition of the current through
     the actual section of this structure (default value: 1 m) */

  Length = 0.001 ;  // (2) thickness of the plate

}


/* --------------------------------------------------------------------------*/

Constraint {

  /* Now, some Dirichlet conditions are defined. The name 
     'ElectricScalarPotential' refers to the constraint name given in
     the function space */

  { Name ElectricScalarPotential ;
    Case { /* A reference must be given for the scalar potential */
      { Region Elec0 ; Value 0. ; }
    }
  }

  // Either the potential is fixed... (1), (2)

  { Name GlobalElectricPotential ;
    Case {
      { Region Elec1 ; Value 1.e-3 ; }
    }
  }

  // ... or the current is fixed (2)

/*
  { Name GlobalElectricCurrent ;
    Case {
      { Region Elec1 ; Value 1. ; }
    }
  }
*/

}

/* The formulations used and their tools, considered as being 
   in a black box, can now be included */

Include "../Jacobian_Lib.pro"
Include "../Integration_Lib.pro"

Include "../EleKin_v.pro"
Include "../EleKin_vf.pro"

/* Finally, we can define some operations to output results */

// (1)

PostOperation Map_v UsingPost EleKin_v {
  Print[ v, OnElementsOf DomainC_Ele, File "Plate_v.pos" ] ;
}

PostOperation Map_j UsingPost EleKin_v {
  Print[ j, OnElementsOf DomainC_Ele, File "Plate_j.pos" ] ;
}

PostOperation Map_jg UsingPost EleKin_v {
  Print[ j, OnPlane { {0.001,0.001,0} {0.179,0.001,0} {0.001,0.089,0} } {36,18}, 
         File "Plate_jg.pos" ] ;
}

// (2)

PostOperation Map_vf_v UsingPost EleKin_vf {
  Print[ v, OnElementsOf DomainC_Ele, File "Plate_v.pos" ] ;
}

PostOperation Map_vf_j UsingPost EleKin_vf {
  Print[ j, OnElementsOf DomainC_Ele, File "Plate_j.pos" ] ;
}

PostOperation Map_vf_jg UsingPost EleKin_vf {
  Print[ j, OnPlane { {0.001,0.001,0} {0.179,0.001,0} {0.001,0.089,0} } {36,18}, 
         File "Plate_jg.pos" ] ;
}

PostOperation Val_V UsingPost EleKin_vf {
  Print[ V, OnRegion Electrode_Ele ] ;
  Print[ I, OnRegion Electrode_Ele ] ;
  Print[ R, OnRegion Electrode_Ele ] ;
}

// For test: post-calculation of a global quantity
// - can be used if the GlobalTerm with I is not included in the Formulation).
// - a function of type vf (in a SubSpace) must be defined.
// - OnRegion is for BasisFunction entity number.
// - the whole domain (DomainC_Ele) is the domain of integration.
PostOperation Ipos UsingPost EleKin_vf {
  Print[ Ipos[DomainC_Ele], OnRegion Elec1 ] ; 
}
