/* Electrokinetic problem: Plate with 3 disks */
// Patrick Dular

/* -------------------------------------------------------------------
   File "Plate.pro"

   This file defines the problem dependent data structures for
   an electrokinetic-thermal coupled problem.

   Resolution:

   Thermal problem in time domain:

   To compute the solution:
     getdp PlateThe -msh PlateWithDisks.msh -solve Thermal_Time -v 1
   To compute post-results:
     getdp PlateThe -msh PlateWithDisks.msh -pos Map_T Time_T

   Thermal problem in steady-state:

   To compute the solution:
     getdp PlateThe -msh PlateWithDisks.msh -solve Thermal_SteadyState -v 1
   To compute post-results:
     getdp PlateThe -msh PlateWithDisks.msh -pos Point_T

   Coupled problems ...

     DomainQ_The is defined as empty, and 
     DomainJoule_The is the whole conducting region (i.e. Region[{DomainC_Ele}])
     (see Either ... or ... in Group{...})

   e.g.
     getdp PlateThe -msh PlateWithDisks.msh -solve EleKinTheNL_v -v 1
     getdp PlateThe -msh PlateWithDisks.msh -pos Map_sigma Map_T Time_T

   ------------------------------------------------------------------- */

/* Geometry (2D)

        +------------------------------------------------------+
        |                              ___                     |
 Elec1  |       Plate                 /   \                    |
      \ |                            |Disk2|                   |
        |               ___           \___/                    |
        |              /   \                                   |
        |             |Disk1|                                  |
        |              \___/                ___                |
        |                                  /   \               |
        |                                 |Disk3|              | \
        |                                  \___/               |  Elec0
        |                                                      |
        +------------------------------------------------------+

  2D elements in: Plate, Disk1, Disk2, Disk3
  1D elements in: Elec0, Elec1
*/

Group {
  Plate = # 101 ;

  Disk1 = # 111 ;  Disk2 = # 112 ;  Disk3 = # 113 ;

  Elec0 = # 300 ;  Elec1 = # 301 ;

  DomainC_Ele = Region[ {Plate, Disk1, Disk2, Disk3} ] ;

  Electrode_Ele = Region[{Elec1}] ;

  // Thermal regions

  SurConv1 = # 401 ; SurConv2 = # 402 ;

  Domain_The = Region[ {Plate, Disk1, Disk2, Disk3} ] ;

  // Either

  DomainJoule_The = Region[ {} ] ;
  DomainQ_The = Region[ {Disk1} ] ;

  // or
  /*
  DomainJoule_The = Region[ {DomainC_Ele} ] ;
  DomainQ_The = Region[ {} ] ;
  */

  SurfaceConv_The = Region[ {SurConv1, SurConv2, Elec0, Elec1} ] ;

}


/* --------------------------------------------------------------------------*/

Function {
  /*
  sigma [ Plate ] = 4.56e6 ; // Pb

  sigma [ #{Disk1, Disk2, Disk3} ] = 5.9e7 ; // Cu
  */

  // The conductivity depends on the temperature T (argument $1)

  T0 = 293. ;

  sigma [ Plate ] = 4.56e6 / ( 1+4e-3*($1-T0) ) ;

  sigma [ #{Disk1, Disk2, Disk3} ] = 5.9e7 / ( 1+4e-3*($1-T0) ) ;


  Thickness = 1. ;


  lambda[ Plate ] = 35 ;
  lambda[ #{Disk1, Disk2, Disk3} ] = 398 ;

  rhoc[ Plate ] = 11373.*130. ;
  rhoc[ #{Disk1, Disk2, Disk3} ] = 8954.*384. ;

  // lambda and rhoc could depend on T, i.e. argument $1


  h[ #{SurConv1, SurConv2, Elec0, Elec1} ] = 10. ;

  I1 = 100./(Pi*0.012^2) ;  R20 = 1.2e-5 ; alpha = .393e-2 ;

  Q_Vol[ Disk1 ] = I1^2*R20*(1+alpha*($1-293.)) ;

  TemperatureConv[] = 293. ;

  // Data for transient study
  Time_Time0 = 0. ;
  Time_TimeMax = 600.*20*3 ;
  Time_DTime[] = (Time_TimeMax-Time_Time0) / 30. ;
  Time_Theta[] = 1. ;

  // Data for iterative loop
  NL_NbrMax = 100 ; NL_Relax[] = 1 ; NL_Eps = 1.e-6 ;

}


/* --------------------------------------------------------------------------*/

Constraint {

  { Name ElectricScalarPotential ;
    Case { /* A reference must be given for the scalar potential */
      { Region Elec0 ; Value 0. ; }
    }
  }

  /* Either the potential is fixed... */

  { Name GlobalElectricPotential ;
    Case {
      { Region Elec1 ; Value 0.1/4 ; }
    }
  }

  /* ... or the current is fixed */

/*
  { Name GlobalElectricCurrent ;
    Case {
      { Region Elec1 ; Value 1. ; }
    }
  }
*/


  { Name Temperature ;
    Case {
      // Init temperature
      { Type Init ; Region Domain_The ; Value 293. ; }
    }
  }

}


Include "../Jacobian_Lib.pro"
Include "../Integration_Lib.pro"

//Include "../EleKin_v.pro"
//Include "../EleKin_vf.pro"
Include "../EleKinThe_v.pro"

PostOperation Map_v UsingPost EleKin_v {
  Print[ v, OnElementsOf DomainC_Ele, File "PlateWithDisks_v.pos" ] ;
}

PostOperation Map_j UsingPost EleKin_v {
  Print[ j, OnElementsOf DomainC_Ele, File "PlateWithDisks_j.pos" ] ;
}

PostOperation Map_jg UsingPost EleKin_v {
  Print[ j, OnPlane { {0.001,0.001,0} {0.179,0.001,0} {0.001,0.089,0} } {36,18}, 
         File "PlateWithDisks_jg.pos" ] ;
}

PostOperation Map_sigma UsingPost EleKin_v {
  Print[ sigma, OnElementsOf #{Plate}, File  "PlateWithDisks_sigma.pos", TimeStep{5} ] ;
  Print[ sigma, OnElementsOf #{Plate}, File >"PlateWithDisks_sigma.pos", TimeStep{10} ] ;
  Print[ sigma, OnElementsOf #{Plate}, File >"PlateWithDisks_sigma.pos", TimeStep{15} ] ;
  Print[ sigma, OnElementsOf #{Plate}, File >"PlateWithDisks_sigma.pos", TimeStep{20} ] ;
}


PostOperation Map_T UsingPost Thermal_T {
  Print[ T, OnElementsOf Domain_The, File "PlateWithDisks_T20.pos", TimeStep{20} ] ;
}


PostOperation Time_T UsingPost Thermal_T {
  Print[ T, OnPoint {0.06,0.045,0.}, File "Tb", Format TimeTable ] ;
} // T(t) at center of Disk1

PostOperation Point_T UsingPost Thermal_T {
  Print[ T, OnPoint {0.06,0.045,0.}, Format Table ] ;
} // T at center of Disk1

